﻿--[[---------------------------------------------------------------------------

Copyright (c) 2008-2017 by K. Scott Piel 
All Rights Reserved

E-mail: < kscottpiel@gmail.com >

This file is part of nUI.

The author no longer develops or directly supports this addon and has released it
into the public domain under the Creative Commons Attribution-NonCommercial 3.0 
Unported license - a copy of which is included in this distribution. This source
may not be distributed without said license.
	   
nUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
PURPOSE.  See the enclosed license for more details.
	
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS  "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT  LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.

--]]---------------------------------------------------------------------------

if not nUI_Profile then nUI_Profile = {}; end;

local GetFactionInfo         = GetFactionInfo;
local GetNumFactions         = GetNumFactions;
local GetWatchedFactionInfo  = GetWatchedFactionInfo;
local SetWatchedFactionIndex = SetWatchedFactionIndex;

nUI_Profile.nUI_FactionBar = {};

local ProfileCounter = nUI_Profile.nUI_FactionBar;

-------------------------------------------------------------------------------

local FactionStandings =
{
	[0] = nUI_L["Unknown"],	
	[1] = nUI_L["Hated"],	
	[2] = nUI_L["Hostile"],	
	[3] = nUI_L["Unfriendly"],	
	[4] = nUI_L["Neutral"],	
	[5] = nUI_L["Friendly"],	
	[6] = nUI_L["Honored"],	
	[7] = nUI_L["Revered"],	
	[8] = nUI_L["Exalted"], 
};

-------------------------------------------------------------------------------

local background    = CreateFrame( "Frame", "nUI_FactionBarBackground", nUI_Dashboard.Anchor );
local frame         = nUI_Bars:createStatusBar( "nUI_FactionBar", nUI_Dashboard.Anchor );
frame.text          = frame:CreateFontString( "$parentLabel", "OVERLAY" );
frame.Super         = {};

frame.updateBar( nil, { 0, 0, 0, 0 } );
frame:EnableMouse( true );
background:SetAllPoints( frame );

local factionInfo = {}
local paragonInfo = {}

-------------------------------------------------------------------------------
-- Faction
function frame.UpdateWatchInfo()
    local faction,standing,min,max,cur,factionID = GetWatchedFactionInfo()
    factionInfo = {}
    factionInfo["Name"] = faction
    factionInfo["Standing"] = standing
    factionInfo["Min"] = min
    factionInfo["Max"] = max
    factionInfo["Cur"] = cur
    factionInfo["ID"] = factionID
    local total = max-min
    local pos = cur-min    
    factionInfo["Percent"] = pos / total
end

-------------------------------------------------------------------------------
-- Paragon
function frame.UpdateParagonInfo(factionID,faction)
    local isParagon = C_Reputation.IsFactionParagon(factionID)
    factionInfo["isParagon"] = false
    if isParagon then
        factionInfo["isParagon"] = true
        local currentValue, threshold, rewardQuestID, hasRewardPending, tooLowLevelForParagon = C_Reputation.GetFactionParagonInfo(factionID)
        local level = math.floor(currentValue/threshold)-(hasRewardPending and 1 or 0)
        local realValue = tonumber(string.sub( currentValue, string.len( level ) + 1 ))

        factionInfo["Min"] = 0
        factionInfo["Cur"] = realValue
        factionInfo["Max"] = threshold        
        factionInfo["Standing"] = level            
        factionInfo["TooLow"] = tooLowLevelForParagon

        local total = threshold
        local pos = realValue    
        factionInfo["Percent"] = pos / total
    end
end    

-------------------------------------------------------------------------------
-- Major 
function frame.UpdateMajorFactionInfo(factionID)    
    local isUnlocked,UnlockDescription = false,nil
    local majorFactionData = C_MajorFactions.GetMajorFactionData(factionID)
    factionInfo["isMajorFaction"] = false
    if majorFactionData then   
        factionInfo["isMajorFaction"] = true
        factionInfo["Name"] = majorFactionData.name        
        factionInfo["Standing"] = majorFactionData.renownLevel
        factionInfo["IsUnlocked"] = majorFactionData.isUnlocked
        factionInfo["UnlockDescription"] = majorFactionData.unlockDescription
        factionInfo["Cur"] = majorFactionData.renownReputationEarned 
        factionInfo["Min"] = 0
        factionInfo["Max"] = majorFactionData.renownLevelThreshold 
        factionInfo["IsMaxed"] = C_MajorFactions.HasMaximumRenown(factionID)
        
        local total = majorFactionData.renownLevelThreshold
        local pos = majorFactionData.renownReputationEarned    
        factionInfo["Percent"] = pos / total
        
    end        
end

-------------------------------------------------------------------------------
-- Friendship 
function frame.UpdateFriendshipInfo(factionID)
    local friendshipData = C_GossipInfo.GetFriendshipReputation(factionID) 
    factionInfo["isFriendship"] = false
    if friendshipData and friendshipData.maxRep > 0 then
        factionInfo["isFriendship"] = true
        factionInfo["Name"] = friendshipData.name
        factionInfo["Standing"] = friendshipData.reaction
        factionInfo["Min"] = friendshipData.reactionThreshold
        factionInfo["Cur"] = friendshipData.standing
        factionInfo["Max"] = friendshipData.nextThreshold
        factionInfo["UsefulInfo"] = friendshipData.text

        local total = friendshipData.nextThreshold - friendshipData.reactionThreshold
        local pos = friendshipData.standing - friendshipData.reactionThreshold
        factionInfo["Percent"] = pos / total
    end
end


-------------------------------------------------------------------------------
-- Update All Faction Info 
function frame.UpdateFactionInfo()
    
	-- who are we actually watching?
    frame.UpdateWatchInfo()
    
    local factionID = factionInfo["ID"]
    local faction = factionInfo["Name"]

    -- Deal with Paragon Factions
    if factionID then
        frame.UpdateParagonInfo(factionID, faction)
    end
        
    -- Deal with Major Factions
    if factionID then
        frame.UpdateMajorFactionInfo(factionID)
    end
        
    -- Deal with Friendships
    if factionID then
        frame.UpdateFriendshipInfo(factionID)
    end
    
end


local function UpdateFaction( msg, factionID )

--	nUI_ProfileStart( ProfileCounter, "UpdateFaction" );
	
	local	track_faction;
	
	-- if we trapped a combat log chat message, then which faction was it talking about?
	
	if msg then

		-- make sure the message is both an update and and increase in
		-- faction (we ignore faction decreases as they usually immediately
		-- follow the increase that we really care about)
		
		if msg:match( nUI_L[nUI_FACTION_UPDATE_START_STRING] )
		and msg:find( nUI_L[nUI_FACTION_UPDATE_INCREASE_STRING] )
		then
			
			local start_idx = nUI_L[nUI_FACTION_UPDATE_START_STRING]:len() + 1;
			local end_idx   = msg:find( nUI_L[nUI_FACTION_UPDATE_END_STRING] );
			local faction   = end_idx and msg:sub( start_idx, end_idx-1 ) or nil;
				
			-- if we have a new faction to track, make it the watched faction
			
			if faction and frame.faction ~= faction then
				
				local firstFaction = IsInGuild() and 3 or 1;
				
                local numFactions = GetNumFactions();
				for i=firstFaction, numFactions do
					
					if faction == GetFactionInfo( i ) then
                        SetWatchedFactionIndex( i );
						break;
					end
				end
			end
        else 
            if factionID then
                C_Reputation.SetWatchedFaction(factionID)
            end
		end
	end

    frame.UpdateFactionInfo()

    -- if we don't have a faction we're watching, then hide the bar
	if not factionInfo["Name"] then
        frame.showing = false;
        if frame.options and frame.options.bar then
            frame.updateBar( nil, frame.options.bar.color );
        end
        frame:SetScript( "OnEnter", nil );
        frame:SetScript( "OnLeave", nil );
		
	else    
	
		if not frame.showing then
			frame.showing = true;

			frame:SetScript( "OnEnter",
				function()
--					nUI_ProfileStart( ProfileCounter, "OnEnter" );
                    frame.UpdateFactionInfo()
					GameTooltip:SetOwner( frame );
					GameTooltip:SetText( nUI_L["Faction: <faction name>"]:format( factionInfo["Name"] ), 1, 0.83, 0 );
                    if factionInfo["isMajorFaction"] then
                        GameTooltip:AddLine( RENOWN_LEVEL_LABEL .. format("|cFF00FFFF%d|r",factionInfo["Standing"]),1,0.83,0)
                        GameTooltip:AddLine( nUI_L["Current Rep: <number>"]:format( factionInfo["Cur"] - factionInfo["Min"] ), 1, 0.83, 0 );
                        GameTooltip:AddLine( nUI_L["Required Rep: <number>"]:format( factionInfo["Max"] - factionInfo["Min"] ), 1, 0.83, 0 );
                        GameTooltip:AddLine( nUI_L["Remaining Rep: <number>"]:format( factionInfo["Max"] - factionInfo["Cur"] ), 1, 0.83, 0 );
                        GameTooltip:AddLine( nUI_L["Percent Complete: <number>"]:format( factionInfo["Percent"] * 100 ), 1, 0.83, 0 );
                        if not factionInfo["IsUnlocked"] then
                            GameTooltip:AddLine( factionInfo["IsUnlocked"],1,0.83,0)
                            GameTooltip:AddLine( factionInfo["UnlockDescription"],1,0.83,0)
                        elseif factionInfo["IsMaxed"] then
                            GameTooltip:AddLine(MAJOR_FACTION_MAX_RENOWN_REACHED,1,0.83,0)
                        end
                        
                    elseif factionInfo["isFriendship"] then
                        GameTooltip:AddLine( factionInfo["UsefulInfo"], 1,0.83,0)
                        GameTooltip:AddLine( nUI_L["Reputation: <rep level>"]:format( factionInfo["Standing"] ), 1, 0.83, 0 );
                        GameTooltip:AddLine( nUI_L["Current Rep: <number>"]:format( factionInfo["Cur"] - factionInfo["Min"] ), 1, 0.83, 0 );
					    GameTooltip:AddLine( nUI_L["Required Rep: <number>"]:format( factionInfo["Max"] - factionInfo["Min"] ), 1, 0.83, 0 );
					    GameTooltip:AddLine( nUI_L["Remaining Rep: <number>"]:format( factionInfo["Max"] - factionInfo["Cur"] ), 1, 0.83, 0 );
                        GameTooltip:AddLine( nUI_L["Percent Complete: <number>"]:format( factionInfo["Percent"] * 100 ), 1, 0.83, 0 );
                        
                    elseif factionInfo["isParagon"] then
                        GameTooltip:AddLine( nUI_L["Reputation: <rep level>"]:format( factionInfo["Standing"] ), 1, 0.83, 0 );
                        GameTooltip:AddLine( nUI_L["Current Rep: <number>"]:format( factionInfo["Cur"] - factionInfo["Min"] ), 1, 0.83, 0 );
					    GameTooltip:AddLine( nUI_L["Required Rep: <number>"]:format( factionInfo["Max"] - factionInfo["Min"] ), 1, 0.83, 0 );
					    GameTooltip:AddLine( nUI_L["Remaining Rep: <number>"]:format( factionInfo["Max"] - factionInfo["Cur"] ), 1, 0.83, 0 );
                        GameTooltip:AddLine( nUI_L["Percent Complete: <number>"]:format( factionInfo["Percent"] * 100 ), 1, 0.83, 0 );
                        if factionInfo["TooLow"] then
                            GameTooltip:AddLine(PARAGON_REPUTATION_TOOLTIP_TEXT_LOW_LEVEL,1,0.83,0)
                        end
                        
                    else
                        GameTooltip:AddLine( nUI_L["Reputation: <rep level>"]:format( FactionStandings[factionInfo["Standing"]] ), 1, 0.83, 0 );
                        GameTooltip:AddLine( nUI_L["Current Rep: <number>"]:format( factionInfo["Cur"] - factionInfo["Min"] ), 1, 0.83, 0 );
                        GameTooltip:AddLine( nUI_L["Required Rep: <number>"]:format( factionInfo["Max"] - factionInfo["Min"] ), 1, 0.83, 0 );
                        GameTooltip:AddLine( nUI_L["Remaining Rep: <number>"]:format( factionInfo["Max"] - factionInfo["Cur"] ), 1, 0.83, 0 );
                        GameTooltip:AddLine( nUI_L["Percent Complete: <number>"]:format( factionInfo["Percent"] * 100 ), 1, 0.83, 0 );
                        
                    end
				    GameTooltip:Show();
--					nUI_ProfileStop();
				end
			);
					
			frame:SetScript( "OnLeave", 
				function() 
--					nUI_ProfileStart( ProfileCounter, "OnLeave" );	
					GameTooltip:Hide(); 
--					nUI_ProfileStop();
				end 
			);
		end

        if frame.enabled and frame.bar.enabled then
            frame.updateBar( tonumber(factionInfo["Percent"]), frame.options.bar.color );
        end
			
        local label = ""            
        if factionInfo["isMajorFaction"] then
            label = nUI_L["<faction name> (<reputation>) <current rep> of <required rep> (<percent complete>)"]:format( factionInfo["Name"], factionInfo["Standing"], factionInfo["Cur"] - factionInfo["Min"], factionInfo["Max"] - factionInfo["Min"], factionInfo["Percent"] * 100 )                
        elseif factionInfo["isFriendship"] then
            label = nUI_L["<faction name> (<reputation>) <current rep> of <required rep> (<percent complete>)"]:format( factionInfo["Name"], factionInfo["Standing"], factionInfo["Cur"] - factionInfo["Min"], factionInfo["Max"] - factionInfo["Min"], factionInfo["Percent"] * 100 )
        elseif factionInfo["isParagon"] then
            label = nUI_L["<faction name> (<reputation>) <current rep> of <required rep> (<percent complete>)"]:format( factionInfo["Name"], factionInfo["Standing"], factionInfo["Cur"] - factionInfo["Min"], factionInfo["Max"] - factionInfo["Min"], factionInfo["Percent"] * 100 )
        else
            label = nUI_L["<faction name> (<reputation>) <current rep> of <required rep> (<percent complete>)"]:format( factionInfo["Name"], FactionStandings[factionInfo["Standing"]], factionInfo["Cur"] - factionInfo["Min"], factionInfo["Max"] - factionInfo["Min"], factionInfo["Percent"] * 100 )
        end

        if frame.text.enabled and frame.text.value ~= label then
            frame.text.value = label;
            frame.text:SetText( label );
            frame.text:SetAlpha(0.5)

        elseif not frame.text.enabled and frame.text.value then
            frame.text.value = nil;
            frame.text:SetText( "" );

        end
    end		
	
--	nUI_ProfileStop();
	
end

-------------------------------------------------------------------------------
-- Allow external function to trigger update
function frame.setActive()
    factionInfo = nil
    UpdateFaction()
end

-------------------------------------------------------------------------------
-- Allow external function to identify whether this bar should be tracked
function frame.isTrackable()
    local associated = frame.barID and frame.barID ~= 0
    return not associated
end

-------------------------------------------------------------------------------
-- Event function
local function onFactionEvent( who, event, arg1 )
	
--	nUI_ProfileStart( ProfileCounter, "onFactionEvent", event );
	
	if event == "ADDON_LOADED" then
		
		if arg1 == "nUI" then		
			nUI:registerScalableFrame( frame );
			nUI:registerSkinnedFrame( frame );
		end
		
	elseif event == "PLAYER_ENTERING_WORLD" then
        if not frame.barID or frame.barID == 0 then
            frame:Hide()
        else
		  UpdateFaction();
        end

	elseif event == "UPDATE_FACTION" then 	
		UpdateFaction();
		
    elseif event == "MAJOR_FACTION_RENOWN_LEVEL_CHANGED" or event == "MAJOR_FACTION_UNLOCKED" then
        UpdateFaction(nil,arg1)
        
    else	
		UpdateFaction( arg1 );
		
	end
	
--	nUI_ProfileStop();
	
end

background:SetScript( "OnEvent", onFactionEvent );
background:RegisterEvent( "ADDON_LOADED" );
background:RegisterEvent( "PLAYER_ENTERING_WORLD" );
background:RegisterEvent( "UPDATE_FACTION" );
background:RegisterEvent( "MAJOR_FACTION_RENOWN_LEVEL_CHANGED")
background:RegisterEvent("MAJOR_FACTION_UNLOCKED")
background:RegisterEvent( "CHAT_MSG_COMBAT_FACTION_CHANGE" );

-------------------------------------------------------------------------------

frame.applyScale = function( scale )
	
--	nUI_ProfileStart( ProfileCounter, "applyScale" );
	
	local anchor  = scale and frame.anchor or nil;
	local scale   = scale or frame.scale or 1;
	local options = frame.options;
	
    -- Dragonflight: Defaults
    local optWidth = options.width  or frame.width
    local optHeight = options.height or frame.height

    
	frame.scale = scale;
	
	if options then
		
		local width       = optWidth * scale * nUI.hScale;
		local height      = optHeight * scale * nUI.vScale;
		local text        = frame.text;
		local font_size   = (options.label and options.label.fontsize or 12) * scale * 1.75 * nUI.vScale;
		local justifyH    = options.label  and options.label.justifyH or "CENTER";
		local justifyV    = options.label  and options.label.justifyV or "MIDDLE";
	
		-- set the bar size
		
		if frame.width  ~= width
		or frame.height ~= height
		then
			
			frame.width  = width;
			frame.height = height;
			
			frame:SetHeight( height );
			frame:SetWidth( width );

			frame.onSizeChanged();
			
		end
		
		-- set the text font size
					
		if text.font_size ~= font_size
		then
			
			-- first time here?
			
			if not text.font_size then 
				text.active  = true;
			end
	
			text.font_size = font_size;
			text:SetFont( nUI_L["font1"], font_size, "OUTLINE" );

		end
		
		-- show or hide the text based on whether or not there is a config for it
		
		text.enabled = options.label and options.label.enabled or false;
		
		if not text.enabled and text.active then

			text.active = false;
			text.value  = nil;				
			text:SetAlpha( 0 );
			text:SetText( "" );				
			
		elseif text.enabled then

			text.active = true;
			text:SetAlpha( 1 );
			
		end
		
		-- set text justification
		
		if text.justifyH ~= justifyH then
			text.justifyH = justifyH;
			text:SetJustifyH( justifyH );
		end
		
		if text.justigyV ~= justifyV then
			text.justifyV = justifyV;
			text:SetJustifyV( justifyV );
		end
	end
	
	if anchor then frame.applyAnchor( anchor ); end

--	nUI_ProfileStop();
	
end

-------------------------------------------------------------------------------

frame.applyAnchor = function( anchor )
	
--	nUI_ProfileStart( ProfileCounter, "applyAnchor" );
	
	local anchor      = anchor or frame.anchor or {};
	local anchor_pt   = anchor.anchor_pt or "CENTER";
    -- Dragonflight TODO
	local relative_to = anchor.relative_to or frame:GetParent():GetName();
	local relative_pt = anchor.relative_pt or anchor_pt;
	local xOfs        = (anchor.xOfs or 0) * nUI.hScale;
	local yOfs        = (anchor.yOfs or 0) * nUI.vScale;
	
	if frame.anchor_pt   ~= anchor_pt
	or frame.relative_to ~= relative_to
	or frame.relative_pt ~= relative_pt
	or frame.xOfs        ~= xOfs
	or frame.yOfs        ~= yOfs
	then
		
		frame.anchor_pt = anchor_pt;
		frame.relative_to = relative_to;
		frame.relative_pt = relative_pt;
		frame.xOfs        = xOfs;
		frame.yOfs        = yOfs;
		
		frame:ClearAllPoints();
		frame:SetPoint( anchor_pt, relative_to, relative_pt, xOfs, yOfs );
		
	end
	
	if frame.options and frame.text.enabled then
		
		local text  = frame.text;
		local label = frame.options.label;

		anchor_pt   = label.anchor_pt or "CENTER";
		relative_to = label.relative_to or frame:GetName();
		relative_pt = label.relative_pt or anchor_pt;
		xOfs        = (label.xOfs or 0) * frame.scale * nUI.hScale;
		yOfs        = (label.yOfs or 0) * frame.scale * nUI.vScale;
		
		if text.anchor_pt ~= anchor_pt
		or text.relative_to ~= relative_to
		or text.relative_pt ~= relative_pt
		or text.xOfs        ~= xOfs
		or text.yOfs        ~= yOfs
		then
			
			text.anchor_pt = anchor_pt;
			text.relative_to = relative_to;
			text.relative_pt = relative_pt;
			text.xOfs        = xOfs;
			text.yOfs        = yOfs;
			
			text:SetPoint( anchor_pt, relative_to, relative_pt, xOfs, yOfs );
			
		end
	end
	
--	nUI_ProfileStop();
	
end

-------------------------------------------------------------------------------

frame.applyOptions = function( options )
	
--	nUI_ProfileStart( ProfileCounter, "applyOptions" );
	
	frame.options = options;
	
	if not options or not options.enabled then

		frame.setEnabled( false );
		frame:Hide();
		
	else
		
		frame.setEnabled( true );
		frame:Show();
	
		background:SetFrameStrata( options.strata or background:GetParent():GetFrameStrata() );
		background:SetFrameLevel( background:GetFrameLevel() + (options.level or 1) );
		
		frame:SetFrameStrata( background:GetFrameStrata() );
		frame:SetFrameLevel( background:GetFrameLevel()+1 );
		
		-- set up the bar
		
		if not options.bar or not options.bar.enabled then
			
			frame.bar.enabled = false;
			frame.updateBar( nil, { 0, 0, 0, 0 } );
			
		else
			
			frame.bar.enabled = true;
			
			frame.setBar( nil, 0, 1 );
			frame.setOverlay( options.bar.overlay );
			frame.setOrientation( options.bar.orient );
			
		end
		
		-- if there's a border, set it
		
		if options.border then
				
			local border_color = options.border.color.border;
			local backdrop_color = options.border.color.backdrop;
			
            frame.Backdrop = CreateFrame("Frame", frame:GetName().."Backdrop", frame, "BackdropTemplate")
            frame.Backdrop:SetAllPoints()

			frame.Backdrop:SetBackdrop( options.border.backdrop );
			frame.Backdrop:SetBackdropBorderColor( border_color.r, border_color.g, border_color.b, border_color.a );
			frame.Backdrop:SetBackdropColor( backdrop_color.r, backdrop_color.g, backdrop_color.b, backdrop_color.a );
	
		else 
			
			--frame:SetBackdrop( nil );
			
		end
		
		-- if there's a background, set it
		
		if options.background then
				
			local border_color = options.background.color.border;
			local backdrop_color = options.background.color.backdrop;
			
           background.Backdrop = CreateFrame("Frame", background:GetName().."Backdrop", background, "BackdropTemplate")
            background.Backdrop:SetAllPoints()
            
			background.Backdrop:SetAlpha( 1 );
			background.Backdrop:SetBackdrop( options.background.backdrop );
			background.Backdrop:SetBackdropBorderColor( border_color.r, border_color.g, border_color.b, border_color.a );
			background.Backdrop:SetBackdropColor( backdrop_color.r, backdrop_color.g, backdrop_color.b, backdrop_color.a );
	
		else 
			
			background:SetAlpha( 0 );
			--background:SetBackdrop( nil );
			
		end
		
		frame.applyScale( options.scale or frame.scale or 1 );
		
	end
	
--	nUI_ProfileStop();
	
end

-------------------------------------------------------------------------------

frame.applySkin = function( skin )
	
--	nUI_ProfileStart( ProfileCounter, "applySkin" );
	
	local skin = skin and skin.FactionBar or nUI_DefaultConfig.FactionBar;
	
	if not skin or not skin.enabled then
		
		frame.setEnabled( false );
		frame:Hide();
		background:Hide();
		
	else
		
		frame.setEnabled( true );
		frame:Show();
		background:Show();
		
		frame.applyOptions( skin.options );
		frame.applyAnchor( skin.anchor );
		
	end
	
--	nUI_ProfileStop();
	
end
